Rem

chris.bitmapfont Module v0.2
Copyright (c) 2010 Christian Klaussner

This software is provided 'as-is', without any express or implied warranty. In no event will the authors 
be held liable For any damages arising from the use of this software.

Permission is granted To anyone To use this software For any purpose, including commercial applications, 
And To alter it And redistribute it freely, subject To the following restrictions:

1. The origin of this software must Not be misrepresented; you must Not claim that you wrote the 
   original software. If you use this software in a product, an acknowledgment in the product 
   documentation would be appreciated but is Not required.

2. Altered source versions must be plainly marked as such, And must Not be misrepresented as being 
   the original software.

3. This notice may Not be removed Or altered from any source distribution.

End Rem

Strict

Module chris.bitmapfont

Import brl.font
Import brl.stream
Import brl.pixmap
Import brl.filesystem

Type TBitmapGlyph Extends TGlyph
	Field pixmap:TPixmap
	Field adv:Float, x, y, width, height
	
	Method Pixels:TPixmap()
		Return pixmap
	End Method
	
	Method Advance:Float()
		Return adv
	End Method
	
	Method GetRect(x:Int Var, y:Int Var, width:Int Var, height:Int Var)
		x = Self.x
		y = Self.y
		width = Self.width
		height = Self.height
	End Method
End Type

Type TBitmapFont Extends TFont
	Field pixmap:TPixmap
	Field glyphs:TBitmapGlyph[224]
	Field style_, height_
	
	Method Style()
		Return style_
	End Method
	
	Method Height()
		Return height_
	End Method
	
	Method CountGlyphs()
		Return glyphs.length
	End Method
	
	Method CharToGlyph(char)
		If char < 255 Return char - 32
		Return 0
	End Method
	
	Method LoadGlyph:TBitmapGlyph(index)
		If glyphs[index] Return glyphs[index]
		Return Null
	End Method
	
	Function LoadFont:TBitmapFont(url:String, size, style)
		Local font:TBitmapFont = New TBitmapFont
		
		Local file:TStream = ReadStream(url)
		If Not file Return Null
		
		If Not (file.ReadByte() = 66 And file.ReadByte() = 70) Return Null
		Local version = file.ReadShort()
		
		Local file_name:String = file.ReadString(file.ReadShort())

		font.pixmap = LoadPixmap(ExtractDir(url) + "/" + file_name)
		If Not font.pixmap
			CloseStream file
			Return Null
		End If
		
		Local h = file.ReadShort()
		Local ascend = file.ReadShort()
		
		For Local i:Int = 0 To 223
			Local dx = file.ReadShort()
			Local dy = file.ReadShort()
			Local width = file.ReadShort()
			Local height = file.ReadShort()
			Local adv = file.ReadShort()
			Local bx = file.ReadInt()
			Local by = file.ReadInt()
			
			font.glyphs[i] = New TBitmapGlyph
			
			Local px = (i Mod 16)*h+dx
			Local py = (i/16)*h+dy
			Local source:Byte Ptr = font.pixmap.PixelPtr(px, py)
			Local pitch = PixmapPitch(font.pixmap)
			Local format = PixmapFormat(font.pixmap)
			
			If width And height
				font.glyphs[i].pixmap = CreateStaticPixmap(source, width, height, pitch, format)
			End If
			
			font.glyphs[i].width = width
			font.glyphs[i].height = height
			font.glyphs[i].adv = adv
			font.glyphs[i].x = bx
			font.glyphs[i].y = -by + ascend
		Next
		
		font.style_ = style
		font.height_ = h
		
		CloseStream file
		
		Return font
	End Function
End Type

Type TBitmapFontLoader Extends TFontLoader
	Method LoadFont:TBitmapFont(url:Object, size, style)
		Local file:String = String(url)
		
		Return TBitmapFont.LoadFont(file, size, style)
	End Method
End Type

AddFontLoader New TBitmapFontLoader
