Rem

chris.bitmapfont Converter v0.1
Copyright (c) 2010 Christian Klaussner

This software is provided 'as-is', without any express or implied warranty. In no event will the authors 
be held liable For any damages arising from the use of this software.

Permission is granted To anyone To use this software For any purpose, including commercial applications, 
And To alter it And redistribute it freely, subject To the following restrictions:

1. The origin of this software must Not be misrepresented; you must Not claim that you wrote the 
   original software. If you use this software in a product, an acknowledgment in the product 
   documentation would be appreciated but is Not required.

2. Altered source versions must be plainly marked as such, And must Not be misrepresented as being 
   the original software.

3. This notice may Not be removed Or altered from any source distribution.

End Rem

Strict

Framework brl.blitz

Import pub.freetype
Import brl.pngloader
Import brl.standardio

Type TConverterGlyph
	Field pixmap:TPixmap
	Field width, height, advance, bx, by
End Type

Type TConverterFont
	Field size, padding, size_padded, advpad
	Field face_:Byte Ptr, face:FTFace = New FTFace
	Field metrics:FTMetrics = New FTMetrics
	Field glyphs:TConverterGlyph[256]
	
	Method LoadFont:TConverterFont(filename:String, size_, padding_, advpad_)
		If FT_New_Face(ftlib, filename, 0, Varptr face_) Return Null
		
		FT_Set_Pixel_Sizes(face_, 0, size_)
		
		MemCopy face, face_, SizeOf(FTFace)
		MemCopy metrics, face.metrics, SizeOf(FTMetrics)
		
		Local slot:FTGlyph = New FTGlyph
		
		For Local i:Int = 0 To 223
			Local index:Int = FT_Get_Char_Index(face_, i+32)
			FT_Load_Glyph(face_, index, FT_LOAD_RENDER)
			
			MemCopy slot, face.glyphslot, SizeOf(FTGlyph)
			
			glyphs[i] = New TConverterGlyph
			glyphs[i].pixmap = CreateStaticPixmap(slot.buffer, slot.width, slot.rows, slot.pitch, PF_A8).Copy()
			glyphs[i].width = slot.width
			glyphs[i].height = slot.rows
			glyphs[i].advance = slot.advancex Sar 6
			glyphs[i].bx = slot.bitmap_left
			glyphs[i].by = slot.bitmap_top
		Next
		
		size = size_
		padding = padding_
		size_padded = size_ + padding_ * 2
		advpad = advpad_
		
		Return Self
	End Method
	
	Method WriteFont(name:String)
		Local outfile:TStream = WriteStream(name + ".font")
		Local outpng:String = name + ".png"
		
		WriteByte outfile, Asc("B")
		WriteByte outfile, Asc("F")
		WriteShort outfile, $0001
		
		WriteShort outfile, Len(outpng)
		WriteString outfile, outpng
		WriteShort outfile, size_padded
		WriteShort outfile, metrics.ascend Sar 6
		
		Local pixmap:TPixmap = CreatePixmap(size_padded * 16, size_padded * 14, PF_A8)
		ClearPixels pixmap
		
		Local x_pos, y_pos:Int
		For Local y:Int = 0 To 13
			x_pos = 0
			
			For Local x:Int = 0 To 15
				Local glyph:TConverterGlyph = glyphs[y*16+x]
				Local dx = (size_padded - (glyph.pixmap.width + padding * 2)) / 2
				Local dy = (size_padded - (glyph.pixmap.height + .padding * 2)) / 2
				
				pixmap.Paste glyph.pixmap, x_pos + padding + dx, y_pos + padding + dy
				x_pos = x_pos + padding * 2 + size
				
				WriteShort outfile, dx
				WriteShort outfile, dy
				WriteShort outfile, glyph.width + padding * 2
				WriteShort outfile, glyph.height + padding * 2
				
				If advpad
					WriteShort outfile, glyph.advance + padding * 2
					WriteInt outfile, glyph.bx
					WriteInt outfile, glyph.by
					
					Else
					
					WriteShort outfile, glyph.advance
					WriteInt outfile, glyph.bx - padding
					WriteInt outfile, glyph.by + padding
				End If
	
			Next
			
			y_pos = y_pos + padding * 2 + size
		Next
		
		CloseStream outfile
		SavePixmapPNG pixmap, outpng
		
	End Method
End Type

Type TParameterParser
	Field params:String[]
	Field index
	
	Method New()
		params = AppArgs
	End Method
	
	Method NextParameter:String()
		index :+ 1
		If index > params.length-1 Return Null
		
		Return params[index]
	End Method
	
	Method Parse(font:String Var, size Var, padding Var, advpad Var, out:String Var)
		Local param:String, param_int
		
		While True
			param = NextParameter()
			If Not param Exit
			
			Select param
				Case "-font"
					param = NextParameter()
					If param font = param Else Print "Error: Missing input file." ; End
				
				Case "-out"
					param = NextParameter()
					If param out = param Else Print "Error: Missing output file." ; End
				
				Case "-size"
					param = NextParameter()
					If Not param Print "Error: Missing size parameter." ; End
					param_int = Int(param)
					If param_int > 0 size = param_int Else Print "Error: Invalid size." ; End
				
				Case "-padding"
					param = NextParameter()
					If Not param Print "Error: Missing padding parameter." ; End
					param_int = Int(param)
					If param_int >= 0 padding = param_int Else Print "Error: Invalid padding." ; End
				
				Case "-advpad"
					advpad = True
				
				Case "-help"
					Print "chris.bitmapfont Converter - (c) Christian Klaussner~n"
					Print "-font <filename> - Filename of the source font."
					Print "-out <name> - Name of the output files. Two files <name>.png and <name>.font will be created."
					Print "-size <pixels> - Font size in pixels."
					Print "-padding <pixels> - Padding in pixels for each character."
					Print "-advpad - Add padding to each characters' advance width."
					Print "-help - Display this message."
					End
				
				Default
					Print "Error: Unknown parameter '" + param + "'."
					End
			End Select
		Wend
	End Method
End Type

Global ftlib:Byte Ptr
Global file:String, out:String = "out"
Global size, padding, size_padded, advpad

Local t:TParameterParser = New TParameterParser
t.Parse file, size, padding, advpad, out
If Not file Print "Error: No input file specified. Type '-help' for more information." ; End

FT_Init_FreeType(Varptr ftlib)

Local font:TConverterFont = New TConverterFont.LoadFont(file, size, padding, advpad)
If Not font Print "Error: Unable to load font." ; End

font.WriteFont out
